
/*	FL01.C 	*/

/*

	FLIST Utility  
	
	Copyright (C) JXL Systems Inc. 1987.  All rights reserved.
	
	Comments about FLIST or inquiries about TextLib may be addressed to

		JXL Systems Inc.
		215 Berkeley Place
		Brooklyn, NY, 11217
		
	or via BIX mail to "jlowenthal".
	

	Source file 1 of 3
	
	FLIST is distributed as freeware.  You are encouraged to copy,
	distribute, and upload it, provided you distribute the entire
	unmodified .ARC file just as you downloaded it.
	
	JXL Systems Inc.'s purpose in distributing FLIST free
	is to illustrate the use of and promote interest in its
	TextLib (R) screen and keyboard management package.

	TextLib is a function library available for the Lattice (R),
	Microsoft (R), Quick C (R) and Turbo (R) MS/DOS C Compilers.  
	It contains a comprehensive set of screen and keyboard
	management functions.  
	
	FLIST is created using the TextLib package.  You are encouraged
	to study the source of FLIST and observe what the various
	TextLib functions accomplish within it.
	
	Unless you own the TextLib package itself, however, you will be
	unable to compile or link FLIST (thus, you cannot modify it),
	since you do not have the file TEXTLIB.H, which is necessary
	for successful compilation, or the TextLib linktime library, which is
	necessary for linking.

	If you do own the TextLib package, you have the ability to modify,
	recompile, and relink FLIST.  See FLIST.DOC for details.  However,
	you cannot legally distribute a modified copy of FLIST or any of
	its components.
	
*/

#include "syshdrs.h"	/* standard compiler headers */
#include "textlib.h"	/* the TextLib header files */

#define	MAIN			/* so that flist.h will generate externals */

#include "flist.h"		/* the FLIST header file */

/***********************************************************************/
/*                                                                     */
/* 	This file contains MAIN, STARTUP, and UTILITY functions            */
/*                                                                     */
/***********************************************************************/


/***********************************************************************/
/*                                                                     */
/*	This is the MAIN function                                          */
/*                                                                     */
/***********************************************************************/

void	main()

{

	startup();      /* FLIST startup function */

	StackAction("mainmenu");	/* TextLib -- prime the action-stack */
	
	while (ActionStackIsNotEmpty())  	/* while action-stack is not empty */
	{
		if (! ExecuteNextAction()) /* TextLib -- execute it */
			break;	/* end the loop if function returns zero */
	}

	/* Prepare for shutdown */

	ScClr();       /* TextLib function to clear the screen */
	CursorOn();	/* TextLib function to turn on the cursor */
	
	/*	End of FLIST application */
	
}

/***********************************************************************/
/*                                                                     */
/*	This is the STARTUP function                                       */
/*                                                                     */
/***********************************************************************/

startup()

{
	
	/*
	Notice the \xff\xc0 terminating the ensuing text-string.  In TextLib,
	a null does not act as a text-terminator (it is the internal
	representation of a newline, in fact); a displayable text-string
	is terminated with the two-byte hex-code 0xff, 0xc0.  The \xff\x84
	and \xff\x80 surrounding "TextLib" causes that word to be displayed
	in high intensity.
	*/

static char copyrite[] = 
" FLIST 1.0 Copyright (C) JXL Systems Inc. 1988 -- \
A \xff\x84TextLib\xff\x80 (R) Application\xff\xc0";

union {
	int (*funcptr)();
	char *dataptr;
} ptrs;

	/*
		Initialize the TextLib screen-management subsystem.
		Request a color display if the hardware supports it.
	*/

	StartConfig(REQ_COLOR);

	CursorOff();		/* Textlib -- turn the cursor off */
	
	/*
		Initialize the TextLib keyboard-management subsystem.
	*/
	
	KeyInit();

	/*
		Take over INT 23H (Ctrl C) and INT 24H (Critical Error Handler) 
	*/

	ptrs.funcptr = x_int23h;
	segregs.ds = FP_SEG(ptrs.dataptr);
	inregs.x.dx = FP_OFF(ptrs.dataptr);
	inregs.h.ah = 0x25;
	inregs.h.al = 0x23;
	intdosx(&inregs,&outregs,&segregs);

	ptrs.funcptr = x_int24h;
	segregs.ds = FP_SEG(ptrs.dataptr);
	inregs.x.dx = FP_OFF(ptrs.dataptr);
	inregs.h.ah = 0x25;
	inregs.h.al = 0x24;
	intdosx(&inregs,&outregs,&segregs);

	/*
		Build default keystroke-processing tables
	*/

	GetGeneralMenuTable();
	SetStEditTable(stedtbl);
	SetFfBrowseTable(ffbrtbl);  
	SetFfEditTable(ffedtbl); 

	/*
		Describe the screen configuration
		
		In the following ConScr macros, the parameters are:
		
		name  	window-name, whose structure indicates the type of window
				(popup or permanent), and the nesting of windows within
				other windows.
		
		x, y	Upper-left corner coordinates relative to the window that
				"contains" the current window.
				
		width, height
		
		colors	e.g., ardcy = red-on-cyan.  adnorm means "use default color
				of the containing window.  If the program runs on monochrome
				equipment, all colors are ignored and everything becomes
				white-on-black.
		
		box		Nullbox means "no box", Stdbox means "surrounded by a box
				drawn with double-line icons.
				
		gutter	Nullgut means "window is NOT surrounded by a black
				visual "channel".
				
	*/

	ConScr("a",0,0,80,25,adnorm,Nullbox,Nullgut); /* entire screen */
		ConScr(CopyrightWindow,0,0,80,1,awhbl,Nullbox,Nullgut);
		ConScr("ab00a",0,1,80,3,awhbl,Nullbox,Nullgut); /* menu, status */
			ConScr(MainMenuWindow,0,0,30,3,awhbl,Stdbox,Nullgut);
			ConScr(StatusWindow_1,31,0,49,1,ardbl,Nullbox,Nullgut);
			ConScr(StatusWindow_2,31,1,49,1,ardbl,Nullbox,Nullgut);
			ConScr(StatusWindow_3,31,2,49,1,ardbl,Nullbox,Nullgut);
		ConScr(FunctionDisplayWindow,0,4,80,3,awhbl,Stdbox,Nullgut);
		ConScr(DirectoryDisplayWindow,0,7,80,13,"\xa4\x06",Stdbox,Nullgut);
		ConScr(BrowseWindow,0,7,80,13,awhbk,Nullbox,Nullgut);
		ConScr(EditWindow,0,7,80,13,awhbk,Nullbox,Nullgut);
		ConScr(InstructionsWindow,0,20,80,5,awhbl,Stdbox,Nullgut); 
			ConScr(InstructionsWindow_1,1,0,77,1,awhbl,Nullbox,Nullgut);
			ConScr(InstructionsWindow_2,1,1,77,1,awhbl,Nullbox,Nullgut);
			ConScr(InstructionsWindow_3,1,2,77,1,awhbl,Nullbox,Nullgut);
		ConScr(SortMenuPopup,18,6,26,6,abkwh,Stdbox,Nullgut);
		ConScr(ActionMenuPopup,19,7,23,13,abkwh,Stdbox,Nullgut);
			ConScr(ActionMenu_1,1,0,20,1,abkwh,Nullbox,Nullgut);
			ConScr(ActionMenu_2,1,1,20,1,abkwh,Nullbox,Nullgut);
			ConScr(ActionMenuWindow,1,3,20,7,abkwh,Nullbox,Nullgut);
		ConScr(CopyToFilenamePopup,20,8,53,4,abkwh,Stdbox,Nullgut);
			ConScr(CopyToFilename_1,1,0,50,1,abkwh,Nullbox,Nullgut);
			ConScr(CopyToFilenameWindow,1,1,50,1,abkwh,Nullbox,Nullgut);
		ConScr(RenameToFilenamePopup,20,8,53,4,abkwh,Stdbox,Nullgut);
			ConScr(RenameToFilename_1,1,0,50,1,abkwh,Nullbox,Nullgut);
			ConScr(RenameToFilenameWindow,1,1,50,1,abkwh,Nullbox,Nullgut);
		ConScr(DirectoryNamePopup,18,5,37,6,abkwh,Stdbox,Nullgut);
			ConScr(DirectoryName_1,1,0,34,1,abkwh,Nullbox,Nullgut);
			ConScr(DirectoryNameWindow,1,1,34,1,abkwh,Nullbox,Nullgut);
			ConScr(DirectoryName_2,1,2,34,1,ardwh,Nullbox,Nullgut);
			ConScr(DirectoryName_3,1,3,34,1,amgwh,Nullbox,Nullgut);
		ConScr(FileInfoPopup,18,6,53,9,abkwh,Stdbox,Nullgut);
			ConScr(FileInfo_1,1,0,50,1,abkwh,Nullbox,Nullgut);
			ConScr(FileInfo_2,1,1,50,1,abkwh,Nullbox,Nullgut);
			ConScr(FileInfo_3,1,3,50,1,abkwh,Nullbox,Nullgut);
			ConScr(FileInfo_4,1,4,50,1,ablwh,Nullbox,Nullgut);
			ConScr(FileInfo_5,1,6,50,1,agrwh,Nullbox,Nullgut);
		ConScr(EditHelpPopup,4,2,72,20,abkwh,Stdbox,Nullgut);
			ConScr(EditHelp_1,1,0,69,1,abkwh,Nullbox,Nullgut); /* edit-help */
			ConScr("a00ja2a",1,1,69,1,abkwh,Nullbox,Nullgut); /* subfields */
			ConScr("a00ja3a",1,16,69,1,abkwh,Nullbox,Nullgut);
			ConScr(EditHelp_2,1,17,69,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaaa",1,2,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaba",1,3,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaca",1,4,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jada",1,5,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaea",1,6,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jafa",1,7,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaga",1,8,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaha",1,9,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaia",1,10,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaja",1,11,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaka",1,12,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jala",1,13,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jama",35,2,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jana",35,3,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaoa",35,4,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00japa",35,5,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaqa",35,6,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jara",35,7,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jasa",35,8,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jata",35,9,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaua",35,10,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00java",35,11,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jawa",35,12,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00jaxa",35,13,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00ja5a",1,14,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00ja6a",35,14,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00ja7a",1,15,33,1,abkwh,Nullbox,Nullgut);
			ConScr("a00ja8a",35,15,33,1,abkwh,Nullbox,Nullgut);
		ConScr(CreateNewFilePopup,20,8,51,4,abkwh,Stdbox,Nullgut);
			ConScr(CreateNewFile_1,1,0,48,1,abkwh,Nullbox,Nullgut);
			ConScr(CreateNewFileWindow,1,1,48,1,abkwh,Nullbox,Nullgut);

	EndConfig();

	/*
		Put "INSTRUCTIONS" at center of top box of bottom screen-panel
	*/
	
	TopBoxTitle(InstructionsWindow,"INSTRUCTIONS\xff\xc0");

	/*
		Describe the menus used in this application
		
		Parameters in the following macros:
		
		MenuInit() --
			table_name (Menu *)
			window_name
			number of choices on each visible row
			  (so that TextLib can format the menu)
			  
		MenuItem() --
			table_name
			prompt (whatever is specified will be video-reversed when 
			  cursored)
			hotkey (key to select choice without cursoring -- 0 = none)
			value (to be returned by MenuSelection() when this choice is made)
			# of spaces to insert to the left of the prompt
			# of spaces to insert to the right of the prompt
			
		MenuSpec() --
			table_name
			Action_name (as used in StackAction())
			service function
			entry function
			exit function
			suspend function
			resume function
			
		GetSpecialMenuTable() --
			table_name
			hotkey_table_name (Proctbl *)
			
	*/

	/* Main menu, displayed on page "ab00aaa", width 2 items per line */

	MenuInit(mainmenu,MainMenuWindow,2);
		MenuItem(mainmenu," Choose directory ",S_C,MainChooseDirectory,1,1);
		MenuItem(mainmenu," Quit ",S_Q,MainQuit,1,1);
	MenuSpec(mainmenu,"mainmenu",mm,mmentry,NULL,NULL,mmres);
	GetSpecialMenuTable(mainmenu,ipmain);

	/* Directory-sorting menu, page "a00aa", width 1 item per line */

	MenuInit(sortmenu,SortMenuWindow,1);
		MenuItem(sortmenu," sort by Filename ",S_F,SortByFilename,1,0);
		MenuItem(sortmenu," sort by eXtension ",S_X,SortByExtension,1,0);
		MenuItem(sortmenu," sort by Date/time ",S_D,SortByDate,1,0);
		MenuItem(sortmenu," sort by siZe ",S_Z,SortBySize,1,0);
	MenuSpec(sortmenu,"sortmenu",sm,smentry,smexit,NULL,NULL);
	GetSpecialMenuTable(sortmenu,ipsort);

	/* File-action selection menu */

	MenuInit(dirmenu,ActionMenuWindow,1);
		MenuItem(dirmenu," see Info ",S_I,FileSeeInfo,0,0); 
		MenuItem(dirmenu," Delete ",S_D,FileDelete,0,0); 
		MenuItem(dirmenu," Copy ",S_C,FileCopy,0,0); 
		MenuItem(dirmenu," Print ",S_P,FilePrint,0,0); 
		MenuItem(dirmenu," Rename ",S_R,FileRename,0,0); 
		MenuItem(dirmenu," Browse ",S_B,FileBrowse,0,0); 
		MenuItem(dirmenu," Edit ",S_E,FileEdit,0,0); 
	MenuSpec(dirmenu,"dirmenu",di,dientry,diexit,diexit,dientry);
	GetSpecialMenuTable(dirmenu,ipdir);

	/*	Done configuring the static menus */
	
	/*
	The directory list is handled as a menu in which a listed
	file may be selected by cursoring to it and pressing ENTER.
	Since the number and content of the menu fields is not known
	until the directory is read-in, this "menu" is initialized
	only skeletally.

	This (scrollable, as are all menus if necessary) menu
	is displayed on page "ad00a", and the filenames are
	listed four abreast.
	*/

	MenuInit(dirlist,DirectoryDisplayWindow,4);
	MenuSpec(dirlist,"dirlist",dl,dlentry,dlexit,dlsusp,dlres);

	/*
		Describe the non-menu "Actions" used in this application
	*/

	/*

		Parameters for FfEditSpec() and FfBrowseSpec() --
		
		table_name (Freeform *)
		Action name (as used in StackAction())
		window_name
		service function
		entry function
		exit function
		suspend function
		resume function
		
	*/
		
	
	/*	File editing */

	FfEditSpec(kedit,"edit",EditWindow,ed,NULL,NULL,NULL,NULL);
	
	/*	File browsing */

	FfBrowseSpec(kbrowse,"browse",BrowseWindow,br,NULL,NULL,NULL,NULL);

	/*

		Parameters for StEditSpecStandalone() --
		
		table_name (Structin *)
		Action name (as used in StackAction())
		window_name
		service function
		entry function
		exit function
		suspend function
		resume function
		max # of chars that may be typed-in, not incl. CR
		
	*/
		
	/* Specifying the to-file in a copy */

	StEditSpecStandalone(kcopy,
	  "ctofile",CopyToFilenameWindow,cf,cpentry,cpexit,NULL,NULL,49);
	
	/* Specifying the to-file in a rename */

	StEditSpecStandalone(krename,
	  "rtofile",RenameToFilenameWindow,rf,rnentry,rnexit,NULL,NULL,49);
	
	/* Specifying the name of a new file to create */

	StEditSpecStandalone(kcreate,
	  "creatnew",CreateNewFileWindow,cn,cnentry,cnexit,NULL,NULL,12);
	
	/* Specifying the name of the desired directory */
	
	StEditSpecStandalone(kdirname,"dirname",DirectoryNameWindow,
					 dn,dninit,dnend,dnend,dninit,31);
		
	/* 
		Initialize certain FLIST internal structures.
		(This is not TextLib-specific code)
	*/
	
	memset((char *)&fmenuent,' ',sizeof(struct fmenuent));
	fmenuent.null = 0;
	
	memset((char *)&fileinfo,' ',sizeof(struct fileinfo));
	fileinfo.ff = 0xff;
	fileinfo.c0 = 0xc0;
	
	memset((char *)&fileihdr,' ',sizeof(struct fileihdr));
	fileihdr.ff = 0xff;
	fileihdr.c0 = 0xc0;
	movmem("Date",fileihdr.date,4);
	movmem("Time",fileihdr.time,4);
	movmem("Size",fileihdr.size,4);
	movmem("Arc",fileihdr.arc,3);
	movmem("Rdonly",fileihdr.rdonly,6);
	
	memset((char *)&fileprt,' ',sizeof(struct fileprt));
	fileprt.nl = '\n';
	fileprt.null = 0;
	
	memset((char *)&filephdr,' ',sizeof(struct filephdr));
	filephdr.nl1 = '\n'; 
	filephdr.nl2 = '\n';
	movmem("NAME",filephdr.name,4);
	movmem("DATE",filephdr.date,4);
	movmem("TIME",filephdr.time,4);
	movmem("SIZE",filephdr.size,4);
	movmem("ARC",filephdr.arc,3);
	movmem("RDONLY",filephdr.rdonly,6);
	filephdr.null = 0;

	/*	Initialize pointer to DOS DTA */

	getdtptr();
		
	/* Display the copyright notice on page "aa00a" */
	
	Display(CopyrightWindow,copyrite);

		
	/* End of startup */
	
	return(1);
	
}

/***********************************************************************/
/*                                                                     */
/*	These are UTILITY functions                                        */
/*                                                                     */
/***********************************************************************/

/*	 

	Function CONFIRM:
	
	This function asks for a yes/no answer from the user.
	It returns 1 if yes and 0 otherwise.
	
*/

int	confirm(prompt)
char *prompt;

{

char *vidsave;
char *reply;
int  reply_value;
int  reply_stat;

	SaveRectangleVideo("a",vidsave,0,0,80,2);
	ClearRectangle("a",0,0,80,2,0x70);
	reply_stat = FastPrompt("a",0,prompt,NULL,1,reply);
	if (reply_stat == FastPromptReplyReceived) 
	{
		reply_value = toupper(reply[0]) == 'Y';
		free(reply);
	}
	else
		reply_value = 0;
    RestoreRectangleVideo("a",vidsave,0,0,80,2);
    return(reply_value);
		
}

/*	Function to get DOS dta address */

void getdtptr()

{

#ifdef MSC4

	inregs.h.ah = 0x2f;	/* DOS function to retrieve DTA address */
	intdosx(&inregs,&outregs,&segregs);
	segread(&segregs);

	ptr_to_dta.components[0] = outregs.x.bx;	/* offset */
	ptr_to_dta.components[1] = segregs.es;		/* segment */

	dtaptr = ptr_to_dta.pointer;

#endif

#ifdef MSC5

	inregs.h.ah = 0x2f;	/* DOS function to retrieve DTA address */
	intdosx(&inregs,&outregs,&segregs);
	segread(&segregs);

	ptr_to_dta.components[0] = outregs.x.bx;	/* offset */
	ptr_to_dta.components[1] = segregs.es;		/* segment */

	dtaptr = ptr_to_dta.pointer;

#endif

#ifdef __TURBOC__
	dtaptr = (struct dta far *)getdta();
#endif

#ifdef LATTC3
	dtaptr = (struct dta far *)getdta();
#endif

}

void clrstat()		/*	Clear fields at upper right of screen */

{

	Clear(StatusWindow_1);
	Clear(StatusWindow_2);
	Clear(StatusWindow_3);
	
}

void menuinst()		/*	Display instructions for a menu */

{

	Display(InstructionsWindow_1, "Use \x1b\x1a to move cursor\xff\xc0");
	Display(InstructionsWindow_2, 
"Press ENTER to select highlighted field\xff\xc0");
	Display(InstructionsWindow_3,
"OR ... enter capitalized letter to make selection.  (Esc = Quit)\xff\xc0");

}

void getfldat()		/*	Set file date & time into proper array */

{
		
	filedt.integer = selfile -> date; /* file date */
	fileyear = (filedt.date).year;	/* extract year */
	fileyear += 80;	/* add base year (1980) */
	filemonth = (filedt.date).month; /* extract month */
	fileday = (filedt.date).day;	/* extract day */

	filedt.integer = selfile -> time; /* file time */
	filehour = (filedt.time).hour;	/* extract hour */
	fileminute = (filedt.time).minute; /* extract minute */
		
	if (filehour >= 12)  /* if PM */
	{
		ampm = 'p';
		file12h = filehour > 12 ? filehour - 12 : filehour;
	}
	else 	/* if AM */
	{
		ampm = 'a';
		file12h = filehour ? filehour : 12;
	}

	fileyear = fileyear > 99 ? 99 : fileyear;
	filemonth = filemonth > 99 ? 99 : filemonth;
	fileday = fileday > 99 ? 99 : fileday;
	file12h = file12h > 99 ? 99 : file12h;
	fileminute = fileminute > 99 ? 99 : fileminute;

}

void fmtdt()		/*	format file date */

{

	sprintf(wkarea,"%02u-%02u-%02u",filemonth,fileday,fileyear);
	
}

void fmttm()		/*	format file time */

{
	
	sprintf(wkarea,"%02u:%02u%c",file12h,fileminute,ampm);

}

void fmtsize()		/*	format file size */

{

	sprintf(wkarea,"%-8ld",(selfile -> size).sizelong);

}
	
void outofmem()

{

	Display(StatusWindow_3, "ABORTING! OUT OF MEMORY!");
	CursorOn();
	exit(1);
	
}

void f1hlp()		/* write edit/browse instruc to bottom of screen */

{

	Display(InstructionsWindow_1,
		"Press F1 for help\xff\xc0");
	Clear(InstructionsWindow_2);
	Clear(InstructionsWindow_3);

}

void toobig()		/*	Error -- maybe edited file is too big?? */

{

			Display(StatusWindow_3,
				"ERROR (file too big??); CR to continue\xff\xc0");
			if (! getch())
				getch();

}

void edbrhelp(help_array)	/* display edit\browse help */

struct ebhelp *help_array;

{

int	i;

	UpPopup(EditHelpPopup);
			
	Display(EditHelp_1,"KEYSTROKES:\xff\xc0");
	Display(EditHelp_2,"Press any key to continue...\xff\xc0");

	for (i = 0; help_array[i].window_name; ++i) 
		Display(help_array[i].window_name, help_array[i].help_text);
			
	if (! getch()) /* waits for user to type any key */
		getch();
			
	DownPopup(EditHelpPopup);
	
}























